from typing import *
import re
import datetime
import telegram
import aiohttp
import royalnet.commands as rc
import royalnet.utils as ru
import royalnet.backpack.tables as rbt
import royalnet.serf.telegram as rst

from ..tables import *


async def to_imgur(imgur_api_key, photosizes: List[telegram.PhotoSize], caption="") -> str:
    # Select the largest photo
    largest_photo = sorted(photosizes, key=lambda p: p.width * p.height)[-1]
    # Get the photo url
    photo_file: telegram.File = await ru.asyncify(largest_photo.get_file)
    # Forward the url to imgur, as an upload
    async with aiohttp.request("post", "https://api.imgur.com/3/upload", data={
        "image": photo_file.file_path,
        "type": "URL",
        "title": "Diario image",
        "description": caption
    }, headers={
        "Authorization": f"Client-ID {imgur_api_key}"
    }) as request:
        response = await request.json()
        if not response["success"]:
            raise rc.CommandError(f"Imgur returned an error in the image upload: {response}")
        return response["data"]["link"]


class DiarioCommand(rc.Command):
    name: str = "diario"

    description: str = "Aggiungi una citazione al Diario."

    syntax = "[!] \"{testo}\" --[autore], [contesto]"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        async with data.session_acm() as session:
            if isinstance(self.serf, rst.TelegramSerf):
                message: telegram.Message = data.message
                reply: telegram.Message = message.reply_to_message
                creator = await data.find_author(session=session, required=True)
                # noinspection PyUnusedLocal
                quoted: Optional[str]
                # noinspection PyUnusedLocal
                text: Optional[str]
                # noinspection PyUnusedLocal
                context: Optional[str]
                # noinspection PyUnusedLocal
                timestamp: datetime.datetime
                # noinspection PyUnusedLocal
                media_url: Optional[str]
                # noinspection PyUnusedLocal
                spoiler: bool
                if creator is None:
                    await data.reply("⚠️ Devi essere registrato a Royalnet per usare questo comando!")
                    return
                if reply is not None:
                    # Get the message text
                    text = reply.text
                    # Check if there's an image associated with the reply
                    photosizes: Optional[List[telegram.PhotoSize]] = reply.photo
                    if photosizes:
                        # Text is a caption
                        text = reply.caption
                        media_url = await to_imgur(self.config["Imgur"]["token"],
                                                   photosizes, text if text is not None else "")
                    else:
                        media_url = None
                    # Ensure there is a text or an image
                    if not (text or media_url):
                        raise rc.InvalidInputError("Il messaggio a cui hai risposto non contiene testo o immagini.")
                    # Find the Royalnet account associated with the sender
                    quoted_tg = await ru.asyncify(session.query(self.alchemy.get(rbt.Telegram))
                                                  .filter_by(tg_id=reply.from_user.id)
                                                  .one_or_none)
                    quoted_account = quoted_tg.user if quoted_tg is not None else None
                    # Find the quoted name to assign
                    quoted_user: telegram.User = reply.from_user
                    quoted = quoted_user.full_name
                    # Get the timestamp
                    timestamp = reply.date
                    # Set the other properties
                    spoiler = False
                    context = None
                else:
                    # Get the current timestamp
                    timestamp = datetime.datetime.now()
                    # Get the message text
                    raw_text = " ".join(args)
                    # Check if there's an image associated with the reply
                    photosizes: Optional[List[telegram.PhotoSize]] = message.photo
                    if photosizes:
                        media_url = await to_imgur(self.config["Imgur"]["token"],
                                                   photosizes, raw_text if raw_text is not None else "")
                    else:
                        media_url = None
                    # Parse the text, if it exists
                    if raw_text:
                        # Pass the sentence through the diario regex
                        match = re.match(
                            r'(!)? *["«‘“‛‟❛❝〝＂`]([^"]+)["»’”❜❞〞＂`] *(?:(?:-{1,2}|—) *([^,]+))?(?:, *([^ ].*))?',
                            raw_text)
                        # Find the corresponding matches
                        if match is not None:
                            spoiler = bool(match.group(1))
                            text = match.group(2)
                            quoted = match.group(3)
                            context = match.group(4)
                        # Otherwise, consider everything part of the text
                        else:
                            spoiler = False
                            text = raw_text
                            quoted = None
                            context = None
                        # Ensure there's a quoted
                        if not quoted:
                            quoted = None
                        if not context:
                            context = None
                        # Find if there's a Royalnet account associated with the quoted name
                        if quoted is not None:
                            quoted_alias = await ru.asyncify(
                                session.query(self.alchemy.get(rbt.Alias))
                                       .filter_by(alias=quoted.lower()).one_or_none
                            )
                        else:
                            quoted_alias = None
                        quoted_account = quoted_alias.user if quoted_alias is not None else None
                    else:
                        text = None
                        quoted = None
                        quoted_account = None
                        spoiler = False
                        context = None
                    # Ensure there is a text or an image
                    if not (text or media_url):
                        raise rc.InvalidInputError("Manca il testo o l'immagine da inserire nel diario.")
                # Create the diario quote
                diario = self.alchemy.get(Diario)(creator=creator,
                                                  quoted_account=quoted_account,
                                                  quoted=quoted,
                                                  text=text,
                                                  context=context,
                                                  timestamp=timestamp,
                                                  media_url=media_url,
                                                  spoiler=spoiler)
                session.add(diario)
                await ru.asyncify(session.commit)
                await data.reply(f"✅ {str(diario)}")
            else:
                # Find the creator of the quotes
                creator = await data.find_author(session=session, required=True)
                # Recreate the full sentence
                raw_text = " ".join(args)
                # Pass the sentence through the diario regex
                match = re.match(r'(!)? *["«‘“‛‟❛❝〝＂`]([^"]+)["»’”❜❞〞＂`] *(?:(?:-{1,2}|—) *([^,]+))?(?:, *([^ ].*))?',
                                 raw_text)
                # Find the corresponding matches
                if match is not None:
                    spoiler = bool(match.group(1))
                    text = match.group(2)
                    quoted = match.group(3)
                    context = match.group(4)
                # Otherwise, consider everything part of the text
                else:
                    spoiler = False
                    text = raw_text
                    quoted = None
                    context = None
                timestamp = datetime.datetime.now()
                # Ensure there is some text
                if not text:
                    raise rc.InvalidInputError("Manca il testo o l'immagine da inserire nel diario.")
                # Or a quoted
                if not quoted:
                    quoted = None
                if not context:
                    context = None
                # Find if there's a Royalnet account associated with the quoted name
                if quoted is not None:
                    quoted_account = await rbt.User.find(self.alchemy, session, quoted)
                else:
                    quoted_account = None
                if quoted_account is None:
                    raise rc.UserError("Il nome dell'autore è ambiguo, quindi la riga non è stata aggiunta.\n"
                                       "Per piacere, ripeti il comando con un nome più specifico!")
                # Create the diario quote
                DiarioT = self.alchemy.get(Diario)
                diario = DiarioT(creator=creator,
                                 quoted_account=quoted_account,
                                 quoted=quoted,
                                 text=text,
                                 context=context,
                                 timestamp=timestamp,
                                 media_url=None,
                                 spoiler=spoiler)
                session.add(diario)
                await ru.asyncify(session.commit)
                await data.reply(f"✅ {str(diario)}")



from typing import *
import royalnet.constellation.api as rca
import royalnet.utils as ru
from ..tables import *


class ApiDiarioAllStar(rca.ApiStar):
    path = "/api/diario/all/v2"

    tags = ["diario"]

    @rca.magic
    async def get(self, data: rca.ApiData) -> ru.JSON:
        """Get all diario entries."""
        entries: List[Diario] = await ru.asyncify(
            data.session
                .query(self.alchemy.get(Diario))
                .order_by(self.alchemy.get(Diario).diario_id)
                .all
        )
        response = [entry.json() for entry in entries]
        return response



from typing import *
import itsdangerous
import aiohttp

from royalnet.backpack import tables as rbt
import royalnet.commands as rc
import royalnet.utils as ru

from .abstract.linker import LinkerCommand
from ..tables import Osu
from ..stars.api_auth_login_osu import ApiAuthLoginOsuStar


class OsuCommand(LinkerCommand):
    name = "osu"

    description = "Connetti e sincronizza il tuo account di osu!"

    @property
    def client_id(self):
        return self.config[self.name]['client_id']

    @property
    def client_secret(self):
        return self.config[self.name]['client_secret']

    @property
    def base_url(self):
        return self.config['base_url']

    @property
    def secret_key(self):
        return self.config['secret_key']

    async def get_updatables_of_user(self, session, user: rbt.User) -> List[Osu]:
        return user.osu

    async def get_updatables(self, session) -> List[Osu]:
        return await ru.asyncify(session.query(self.alchemy.get(Osu)).all)

    async def create(self,
                     session,
                     user: rbt.User,
                     args: rc.CommandArgs,
                     data: Optional[rc.CommandData] = None) -> Optional[Osu]:
        serializer = itsdangerous.URLSafeSerializer(self.secret_key, salt="osu")
        # TODO: Ensure the chat the link is being sent in is secure!!!
        await data.reply("🔑 [b]Login necessario[/b]\n"
                         f"[url=https://osu.ppy.sh/oauth/authorize"
                         f"?client_id={self.client_id}"
                         f"&redirect_uri={self.base_url}{ApiAuthLoginOsuStar.path}"
                         f"&response_type=code"
                         f"&state={serializer.dumps(user.uid)}]"
                         f"Connetti account di osu! a {user.username}"
                         f"[/url]")
        return None

    async def update(self, session, obj: Osu, change: Callable[[str, Any], Awaitable[None]]):
        await obj.refresh_if_expired(client_id=self.client_id,
                                     client_secret=self.client_secret)
        async with aiohttp.ClientSession(headers={"Authorization": f"Bearer {obj.access_token}"}) as session:
            async with session.get("https://osu.ppy.sh/api/v2/me/osu") as response:
                m = await response.json()
                obj.avatar_url = m["avatar_url"]
                obj.username = m["username"]
                if "statistics" in m:
                    await change("standard_pp", m["statistics"].get("pp"))
            async with session.get("https://osu.ppy.sh/api/v2/me/taiko") as response:
                m = await response.json()
                if "statistics" in m:
                    await change("taiko_pp", m["statistics"].get("pp"))
            async with session.get("https://osu.ppy.sh/api/v2/me/fruits") as response:
                m = await response.json()
                if "statistics" in m:
                    await change("catch_pp", m["statistics"].get("pp"))
            async with session.get("https://osu.ppy.sh/api/v2/me/mania") as response:
                m = await response.json()
                if "statistics" in m:
                    await change("mania_pp", m["statistics"].get("pp"))

    async def on_increase(self, session, obj: Osu, attribute: str, old: Any, new: Any) -> None:
        if attribute == "standard_pp":
            await self.notify(f"📈 [b]{obj.user}[/b] è salito a [b]{new:.0f}pp[/b] su [i]osu![/i]! Congratulazioni!")
        elif attribute == "taiko_pp":
            await self.notify(f"📈 [b]{obj.user}[/b] è salito a [b]{new:.0f}pp[/b] su [i]osu!taiko[/i]! Congratulazioni!")
        elif attribute == "catch_pp":
            await self.notify(f"📈 [b]{obj.user}[/b] è salito a [b]{new:.0f}pp[/b] su [i]osu!catch[/i]! Congratulazioni!")
        elif attribute == "mania_pp":
            await self.notify(f"📈 [b]{obj.user}[/b] è salito a [b]{new:.0f}pp[/b] su [i]osu!mania[/i]! Congratulazioni!")

    async def on_unchanged(self, session, obj: Osu, attribute: str, old: Any, new: Any) -> None:
        pass

    async def on_decrease(self, session, obj: Osu, attribute: str, old: Any, new: Any) -> None:
        if attribute == "standard_pp":
            await self.notify(f"📉 [b]{obj.user}[/b] è sceso a [b]{new:.0f}pp[/b] su [i]osu![/i].")
        elif attribute == "taiko_pp":
            await self.notify(f"📉 [b]{obj.user}[/b] è sceso a [b]{new:.0f}pp[/b] su [i]osu!taiko[/i].")
        elif attribute == "catch_pp":
            await self.notify(f"📉 [b]{obj.user}[/b] è sceso a [b]{new:.0f}pp[/b] su [i]osu!catch[/i].")
        elif attribute == "mania_pp":
            await self.notify(f"📉 [b]{obj.user}[/b] è sceso a [b]{new:.0f}pp[/b] su [i]osu!mania[/i].")

    async def on_first(self, session, obj: Osu, attribute: str, old: None, new: Any) -> None:
        if attribute == "standard_pp":
            await self.notify(f"⭐️ [b]{obj.user}[/b] ha guadagnato i suoi primi [b]{new:.0f}pp[/b] su [i]osu![/i]!")
        elif attribute == "taiko_pp":
            await self.notify(f"⭐️ [b]{obj.user}[/b] ha guadagnato i suoi primi [b]{new:.0f}pp[/b] su [i]osu!taiko[/i]!")
        elif attribute == "catch_pp":
            await self.notify(f"⭐️ [b]{obj.user}[/b] ha guadagnato i suoi primi [b]{new:.0f}pp[/b] su [i]osu!catch[/i]!")
        elif attribute == "mania_pp":
            await self.notify(f"⭐️ [b]{obj.user}[/b] ha guadagnato i suoi primi [b]{new:.0f}pp[/b] su [i]osu!mania[/i]!")

    async def on_reset(self, session, obj: Osu, attribute: str, old: Any, new: None) -> None:
        if attribute == "standard_pp":
            await self.notify(f"⬜️ [b]{obj.user}[/b] non è più classificato su [i]osu![/i].")
        elif attribute == "taiko_pp":
            await self.notify(f"⬜️ [b]{obj.user}[/b] non è più classificato su [i]osu!taiko[/i].")
        elif attribute == "catch_pp":
            await self.notify(f"⬜️ [b]{obj.user}[/b] non è più classificato su [i]osu!catch[/i].")
        elif attribute == "mania_pp":
            await self.notify(f"⬜️ [b]{obj.user}[/b] non è più classificato su [i]osu!mania[/i].")

    def describe(self, obj: Osu) -> str:
        message = [
            f"ℹ️ [url=https://osu.ppy.sh/users/{obj.osu_id}]{obj.username}[/url]",
            f"osu!: [b]{obj.standard_pp:.0f}pp[/b]",
            f"osu!taiko: [b]{obj.taiko_pp:.0f}pp[/b]",
            f"osu!catch: [b]{obj.catch_pp:.0f}pp[/b]",
            f"osu!mania: [b]{obj.mania_pp:.0f}pp[/b]",
        ]
        return "\n".join(message)



from typing import *
import royalnet.constellation.api as rca
import royalnet.utils as ru
from ..tables import *


class ApiDiarioRangeStar(rca.ApiStar):
    path = "/api/diario/range/v3"

    tags = ["diario"]

    parameters = {
        "get": {
            "start": "The starting diario_id (included).",
            "end": "The final diario_id (excluded).",
        }
    }

    @rca.magic
    async def get(self, data: rca.ApiData) -> ru.JSON:
        """Get the diario entries in the specified range."""
        DiarioT = self.alchemy.get(Diario)

        entries: List[Diario] = await ru.asyncify(
            data.session
                .query(DiarioT)
                .filter(
                    DiarioT.diario_id >= data.int("start", optional=False),
                    DiarioT.diario_id < data.int("end", optional=False)
                )
                .order_by(DiarioT.diario_id)
                .all
        )
        response = [entry.json() for entry in entries]
        return response
